%remove transitions from system.
%syntax is :remove_trans(sys,a,b,array) to remove transition a->b and put
%the result in array

function result = remove_trans(sys,a,b,array)

T = sys;
T.formula = ['!(' sys.formula ')'];
if (model_check(T)) %there are no runs satisfying the formula (negation is satisfied) can't refine
    result = {};
    return;
end;

%printf('removing transition (%i->%i)\n',a,b);

result = sys;

result.trans(a,b)=0; %force the transition to 0 (so its not recomputed)

% set param for 'a' to the intersection of the old parameters and
% the safe parameters

param_index = get(result.space(a),'param');
P = intersect(sys.parameter{param_index},param_range_rev(result.space(a),result.space(b)));


states = a;
if (length(P)~=0) %if it is not blocking (has some parameters)
    result.parameter{param_index} = P;
    for (j=1:length(result.space)) %check all polytopes that use the same parameters
        if (get(sys.space(j),'param')==param_index)
            r = find(sys.trans(j,:)); %store all transitions a->r(i)
            result.trans(j,:) = zeros(1,size(result.trans,2)); %make the state blocking
            for(i=1:length(r)) %re-compute all old transitions to see if they still exist
                if (dointersect(Post(result,j),result.space(r(i))))
                        result.trans(j,r(i)) = 1; %add the transitions that still exist
                end;
            end
        end;
    end;
else
    result.parameter{param_index} = {}; %the parameter set is empty (delete it
    for (i=1:length(result.space))
        if (get(result.space(i),'param')==param_index) %if it shares the same parameters
            result.trans(i,:) = zeros(1,size(result.trans,2));
            states = [states, i];
        end;
    end;
end;

result = fix_block(result,states); %fix potential blocking state occuring in a
if (isempty(result) | sum(result.trans(result.init,:))==0 | in(array,result)) %if the initial state is blocking or it's in the array
%    printf('Returning empty (%i,%i)\n',sum(result.trans(result.init,:)),in(array,result));
	result = {};
end;
